# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
from datetime import timedelta
from dateutil.parser import parse
import mock
import pytz

from almanach.core.controllers import volume_controller
from almanach.core import exception
from almanach.core import model
from almanach.storage.drivers import base_driver

from almanach.tests.unit import base
from almanach.tests.unit.builders.entity import a
from almanach.tests.unit.builders.entity import volume
from almanach.tests.unit.builders.entity import volume_type


class TestVolumeController(base.BaseTestCase):

    def setUp(self):
        super(TestVolumeController, self).setUp()
        self.database_adapter = mock.Mock(spec=base_driver.BaseDriver)
        self.controller = volume_controller.VolumeController(self.config, self.database_adapter)

    def test_volume_closed(self):
        fake_volume = a(volume())
        self.database_adapter.count_entity_entries.return_value = 2
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)
        date = date + timedelta(1)
        expected_date = pytz.utc.localize(date)

        self.controller.delete_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))

        self.database_adapter.count_entity_entries.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_volume.entity_id, expected_date)

    def test_volume_deleted_within_volume_existance_threshold(self):
        fake_volume = a(volume())
        self.database_adapter.count_entity_entries.return_value = 2
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)
        date = date + timedelta(0, 5)

        self.controller.delete_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))

        self.database_adapter.count_entity_entries.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.delete_active_entity.assert_called_once_with(fake_volume.entity_id)

    def test_volume_deleted_within_volume_existance_threshold_but_with_only_one_entry(self):
        fake_volume = a(volume())
        self.database_adapter.count_entity_entries.return_value = 1

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)
        date = date + timedelta(0, 5)
        expected_date = pytz.utc.localize(date)

        self.controller.delete_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%fZ"))

        self.database_adapter.count_entity_entries.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_volume.entity_id, expected_date)

    def test_list_volumes(self):
        self.database_adapter.get_all_entities_by_project.return_value = ["volume2", "volume3"]

        self.assertEqual(self.controller.list_volumes("project_id", "start", "end"), ["volume2", "volume3"])

        self.database_adapter.get_all_entities_by_project.assert_called_once_with(
            "project_id", "start", "end", model.Volume.TYPE
        )

    def test_create_volume(self):
        some_volume_type = a(volume_type().with_volume_type_name("some_volume_type_name"))
        self.database_adapter.get_volume_type.return_value = some_volume_type
        self.database_adapter.get_active_entity.return_value = None

        some_volume = a(volume()
                        .with_volume_type(some_volume_type.volume_type_name)
                        .with_all_dates_in_string())

        expected_volume = a(volume()
                            .with_volume_type(some_volume_type.volume_type_name)
                            .with_project_id(some_volume.project_id)
                            .with_id(some_volume.entity_id))

        self.controller.create_volume(some_volume.entity_id, some_volume.project_id, some_volume.start,
                                      some_volume_type.volume_type_id, some_volume.size, some_volume.name,
                                      some_volume.attached_to)

        self.database_adapter.get_volume_type.assert_called_once_with(some_volume_type.volume_type_id)
        self.database_adapter.insert_entity.assert_called_once_with(expected_volume)
        self.database_adapter.get_active_entity.assert_called_once_with(some_volume.entity_id)

    def test_create_volume_raises_bad_date_format(self):
        some_volume = a(volume())

        self.assertRaises(
            exception.DateFormatException,
            self.controller.create_volume,
            some_volume.entity_id,
            some_volume.project_id,
            'bad_date_format',
            some_volume.volume_type,
            some_volume.size,
            some_volume.name,
            some_volume.attached_to
        )

    def test_create_volume_insert_none_volume_type_as_type(self):
        some_volume_type = a(volume_type().with_volume_type_id(None).with_volume_type_name(None))
        self.database_adapter.get_active_entity.return_value = None

        some_volume = a(volume()
                        .with_volume_type(some_volume_type.volume_type_name)
                        .with_all_dates_in_string())

        expected_volume = a(volume()
                            .with_volume_type(some_volume_type.volume_type_name)
                            .with_project_id(some_volume.project_id)
                            .with_id(some_volume.entity_id))

        self.controller.create_volume(some_volume.entity_id, some_volume.project_id, some_volume.start,
                                      some_volume_type.volume_type_id, some_volume.size, some_volume.name,
                                      some_volume.attached_to)

        self.database_adapter.get_volume_type.assert_not_called()
        self.database_adapter.insert_entity.assert_called_once_with(expected_volume)
        self.database_adapter.get_active_entity.assert_called_once_with(some_volume.entity_id)

    def test_create_volume_with_invalid_volume_type(self):
        some_volume_type = a(volume_type())
        self.database_adapter.get_volume_type.side_effect = exception.VolumeTypeNotFoundException
        self.database_adapter.get_active_entity.return_value = None

        some_volume = a(volume()
                        .with_volume_type(some_volume_type.volume_type_name)
                        .with_all_dates_in_string())

        self.assertRaises(exception.VolumeTypeNotFoundException, self.controller.create_volume,
                          some_volume.entity_id, some_volume.project_id, some_volume.start,
                          some_volume_type.volume_type_id, some_volume.size, some_volume.name,
                          some_volume.attached_to)

        self.database_adapter.get_volume_type.assert_called_once_with(some_volume_type.volume_type_id)
        self.database_adapter.insert_entity.assert_not_called()
        self.database_adapter.get_active_entity.assert_called_once_with(some_volume.entity_id)

    def test_create_volume_but_its_an_old_event(self):
        some_volume = a(volume().with_last_event(pytz.utc.localize(datetime(2015, 10, 21, 16, 29, 0))))
        self.database_adapter.get_active_entity.return_value = some_volume

        self.controller.create_volume(some_volume.entity_id, some_volume.project_id, '2015-10-21T16:25:00.000000Z',
                                      some_volume.volume_type, some_volume.size, some_volume.name,
                                      some_volume.attached_to)

        self.database_adapter.get_active_entity.assert_called_once_with(some_volume.entity_id)

    def test_volume_updated(self):
        fake_volume = a(volume())
        dates_str = "2015-10-21T16:25:00.000000Z"
        fake_volume.size = "new_size"
        fake_volume.start = parse(dates_str)
        fake_volume.end = None
        fake_volume.last_event = parse(dates_str)
        self.database_adapter.get_active_entity.return_value = fake_volume

        self.controller.resize_volume(fake_volume.entity_id, "new_size", dates_str)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_volume.entity_id, parse(dates_str))
        self.database_adapter.insert_entity.assert_called_once_with(fake_volume)

    def test_volume_attach_with_no_existing_attachment(self):
        fake_volume = a(volume()
                        .with_no_attachment())
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)

        self.controller.attach_volume(
            fake_volume.entity_id,
            date.strftime("%Y-%m-%dT%H:%M:%S.%f"),
            ["new_attached_to"]
        )

        self.assertEqual(fake_volume.attached_to, ["new_attached_to"])
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.update_active_entity.assert_called_once_with(fake_volume)

    def test_volume_attach_with_existing_attachments(self):
        fake_volume = a(volume()
                        .with_attached_to(["existing_attached_to"]))
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)

        self.controller.attach_volume(
            fake_volume.entity_id,
            date.strftime("%Y-%m-%dT%H:%M:%S.%f"),
            ["existing_attached_to", "new_attached_to"]
        )

        self.assertEqual(fake_volume.attached_to, ["existing_attached_to", "new_attached_to"])
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.update_active_entity.assert_called_once_with(fake_volume)

    def test_volume_attach_after_threshold(self):
        fake_volume = a(volume())
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)
        date = date + timedelta(0, 120)
        expected_date = pytz.utc.localize(date)

        new_volume = a(volume()
                       .build_from(fake_volume)
                       .with_datetime_start(expected_date)
                       .with_no_end()
                       .with_last_event(expected_date)
                       .with_attached_to(["new_attached_to"]))

        self.controller.attach_volume(
            fake_volume.entity_id,
            date.strftime("%Y-%m-%dT%H:%M:%S.%f"),
            ["new_attached_to"]
        )

        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_volume.entity_id, expected_date)
        self.database_adapter.insert_entity.assert_called_once_with(new_volume)

    def test_volume_detach_with_two_attachments(self):
        fake_volume = a(volume().with_attached_to(["I1", "I2"]))
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)

        self.controller.detach_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%fZ"), ["I2"])

        self.assertEqual(fake_volume.attached_to, ["I2"])
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.update_active_entity.assert_called_once_with(fake_volume)

    def test_volume_detach_with_one_attachments(self):
        fake_volume = a(volume().with_attached_to(["I1"]))
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)

        self.controller.detach_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%f"), [])

        self.assertEqual(fake_volume.attached_to, [])
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.update_active_entity.assert_called_once_with(fake_volume)

    def test_volume_detach_last_attachment_after_threshold(self):
        fake_volume = a(volume().with_attached_to(["I1"]))
        self.database_adapter.get_active_entity.return_value = fake_volume

        date = datetime(fake_volume.start.year, fake_volume.start.month, fake_volume.start.day, fake_volume.start.hour,
                        fake_volume.start.minute, fake_volume.start.second, fake_volume.start.microsecond)
        date = date + timedelta(0, 120)
        expected_date = pytz.utc.localize(date)

        new_volume = a(volume()
                       .build_from(fake_volume)
                       .with_datetime_start(expected_date)
                       .with_no_end()
                       .with_last_event(expected_date)
                       .with_no_attachment())

        self.controller.detach_volume(fake_volume.entity_id, date.strftime("%Y-%m-%dT%H:%M:%S.%f"), [])

        self.assertEqual(fake_volume.attached_to, [])
        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.close_active_entity.assert_called_once_with(fake_volume.entity_id, expected_date)
        self.database_adapter.insert_entity.assert_called_once_with(new_volume)

    def test_rename_volume(self):
        fake_volume = a(volume().with_display_name('old_volume_name'))
        volume_name = 'new_volume_name'
        self.database_adapter.get_active_entity.return_value = fake_volume

        new_volume = a(volume().build_from(fake_volume).with_display_name(volume_name))

        self.controller.rename_volume(fake_volume.entity_id, volume_name)

        self.database_adapter.get_active_entity.assert_called_once_with(fake_volume.entity_id)
        self.database_adapter.update_active_entity.assert_called_once_with(new_volume)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from almanach.core.controllers import base_controller
from almanach.core import model

LOG = log.getLogger(__name__)


class VolumeTypeController(base_controller.BaseController):

    def __init__(self, database_adapter):
        self.database_adapter = database_adapter

    def create_volume_type(self, volume_type_id, volume_type_name):
        LOG.info("volume type %s with name %s created", volume_type_id, volume_type_name)
        volume_type = model.VolumeType(volume_type_id, volume_type_name)
        self.database_adapter.insert_volume_type(volume_type)

    def get_volume_type(self, volume_type_id):
        return self.database_adapter.get_volume_type(volume_type_id)

    def delete_volume_type(self, volume_type_id):
        self.database_adapter.delete_volume_type(volume_type_id)

    def list_volume_types(self):
        return self.database_adapter.list_volume_types()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import pytz

from almanach.collector import notification


class NotificationMessageBuilder(object):

    def __init__(self):
        self.event_type = None
        self.context = dict()
        self.payload = dict()
        self.metadata = dict()

    def with_event_type(self, event_type):
        self.event_type = event_type
        return self

    def with_context_value(self, key, value):
        self.context[key] = value
        return self

    def with_payload_value(self, key, value):
        self.payload[key] = value
        return self

    def build(self):
        return notification.NotificationMessage(self.event_type, self.context, self.payload, self.metadata)


class InstanceNotificationBuilder(NotificationMessageBuilder):

    def __init__(self):
        super(InstanceNotificationBuilder, self).__init__()
        self.payload = {
            'instance_id': 'my_instance_id',
            'tenant_id': 'my_tenant_id',
            'created_at': datetime(2014, 2, 14, 16, 29, 58, tzinfo=pytz.utc),
            'terminated_at': None,
            'instance_type': 'my_flavor_name',
            'image_meta': {},
            'hostname': 'my_instance_name',
            'metadata': {},
        }

    def with_image_meta(self, key, value):
        self.payload['image_meta'][key] = value
        return self


class VolumeTypeNotificationBuilder(NotificationMessageBuilder):

    def __init__(self):
        super(VolumeTypeNotificationBuilder, self).__init__()
        self.payload = {
            'volume_types': {
                'name': 'my_volume_type_name',
                "qos_specs_id": None,
                "deleted": False,
                "created_at": "2014-02-14T17:18:35.036186Z",
                "extra_specs": {},
                "deleted_at": None,
                "id": 'my_volume_type_id',
            }
        }


class VolumeNotificationBuilder(NotificationMessageBuilder):

    def __init__(self):
        super(VolumeNotificationBuilder, self).__init__()
        self.payload = {
            'created_at': '2015-07-27T16:11:07Z',
            'tenant_id': 'my_tenant_id',
            'volume_id': 'my_volume_id',
            'display_name': 'my_volume_name',
            'volume_type': 'my_volume_type',
            'size': 1,
            'volume_attachment': [],
        }

    def with_instance_attached(self, instance_id):
        self.payload['volume_attachment'].append({'instance_uuid': instance_id})
        return self



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from tempest.common import waiters
from tempest.lib.common.utils import test_utils

from almanach.tests.tempest.tests.scenario import base


class TestVolumeAttachmentScenario(base.BaseAlmanachScenarioTest):
    _server = None
    _volume = None

    def tearDown(self):
        super(TestVolumeAttachmentScenario, self).tearDown()
        self.addCleanup(waiters.wait_for_server_termination,
                        self.os.servers_client, self._server['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.os.servers_client.delete_server, self._server['id'])
        self.addCleanup(self.volumes_client.wait_for_resource_deletion,
                        self._volume['id'])
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.volumes_client.delete_volume, self._volume['id'])

    def test_attachment(self):
        self._attach_volume_to_server()
        self.wait_for_notification(self._check_that_volume_is_attached)

        entities = self.get_tenant_entities(self._volume['os-vol-tenant-attr:tenant_id'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNone(entities[0]['end'])
        self.assertTrue(self._server['id'] in entities[0]['attached_to'])

        self._detach_volume_from_server()

        self.wait_for_notification(self._check_that_volume_is_detached)

        entities = self.get_tenant_entities(self._volume['os-vol-tenant-attr:tenant_id'])
        self.assertIsNotNone(entities[0]['start'])
        self.assertIsNone(entities[0]['end'])
        self.assertFalse(self._server['id'] in entities[0]['attached_to'])

    def _check_that_volume_is_attached(self):
        entities = self.get_tenant_entities(self._volume['os-vol-tenant-attr:tenant_id'])
        return len(entities) == 1 and self._server['id'] in entities[0]['attached_to']

    def _check_that_volume_is_detached(self):
        entities = self.get_tenant_entities(self._volume['os-vol-tenant-attr:tenant_id'])
        return len(entities) == 1 and self._server['id'] not in entities[0]['attached_to']

    def _attach_volume_to_server(self):
        self._volume = self.create_test_volume()
        self._server, _ = self.create_test_server(wait_until='ACTIVE')

        self.volumes_client.attach_volume(self._volume['id'],
                                          instance_uuid=self._server['id'],
                                          mountpoint='/dev/vdc')

        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self._volume['id'], 'in-use')

    def _detach_volume_from_server(self):
        self.volumes_client.detach_volume(self._volume['id'])
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                self._volume['id'], 'available')



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging

from almanach.collector.filters import base_filter
from almanach.core.helpers import date_helper

LOG = logging.getLogger(__name__)


class DeleteInstanceBeforeCompletionFilter(base_filter.BaseFilter):

    def __init__(self, config):
        self.config = config

    def ignore_notification(self, notification):
        delta = self.config.entities.instance_existence_threshold

        if self._has_been_retried(notification) \
                and self._is_instance_deleted(notification) \
                and self._was_never_created_successfully(notification, delta):
            LOG.info('Instance %s was never created successfully during %d seconds',
                     notification.payload.get('instance_id'),
                     delta)
            return True
        return False

    @staticmethod
    def _is_instance_deleted(notification):
        return notification.payload.get('state') == 'deleted'

    def _has_been_retried(self, notification):
        return notification.get_retry_counter() >= self.config.collector.min_retries

    @staticmethod
    def _was_never_created_successfully(notification, delta):
        creation_date = date_helper.DateHelper().parse(notification.payload.get('created_at'))
        deletion_date = date_helper.DateHelper().parse(notification.payload.get('deleted_at'))
        return date_helper.DateHelper().is_within_range(creation_date, deletion_date, delta)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from oslo_serialization import jsonutils as json
from tempest.common import compute
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions
from tempest.scenario import manager

from almanach.tests.tempest import clients

CONF = config.CONF


class BaseAlmanachScenarioTest(manager.ScenarioTest):
    credentials = ['primary', 'admin']
    notification_interval = 1
    notification_timeout = 30

    @classmethod
    def setup_clients(cls):
        super(BaseAlmanachScenarioTest, cls).setup_clients()
        cred_provider = cls._get_credentials_provider()
        credentials = cred_provider.get_creds_by_roles(['admin']).credentials
        cls.os = clients.Manager(credentials=credentials)
        cls.almanach_client = cls.os.almanach_client

    def get_tenant_entities(self, tenant_id):
        resp, response_body = self.almanach_client.get_tenant_entities(tenant_id)
        self.assertEqual(resp.status, 200)

        response_body = json.loads(response_body)
        self.assertIsInstance(response_body, list)
        return response_body

    def create_test_volume_type(self):
        client = self.os_adm.volume_types_v2_client
        randomized_name = data_utils.rand_name('scenario-volume-type')

        volume_type = client.create_volume_type(name=randomized_name)['volume_type']

        self.assertIn('id', volume_type)
        self.addCleanup(client.delete_volume_type, volume_type['id'])

        return volume_type

    def create_test_volume(self, size=None, volume_type=None):
        name = data_utils.rand_name(self.__class__.__name__)

        if size is None:
            size = CONF.volume.volume_size

        kwargs = {
            'display_name': name,
            'volume_type': volume_type,
            'size': size
        }

        volume = self.volumes_client.create_volume(**kwargs)['volume']
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')

        return self.volumes_client.show_volume(volume['id'])['volume']

    def create_test_server(self, wait_until=None):
        flavors = self.flavors_client.list_flavors()['flavors']
        images = self.image_client.list_images()['images']
        tenant_network = self.get_tenant_network()
        body, servers = compute.create_test_server(
            self.os,
            wait_until=wait_until,
            image_id=images[0]['id'],
            flavor=flavors[0]['id'],
            tenant_network=tenant_network)

        server = self.os.servers_client.show_server(body['id'])['server']
        return server, flavors[0]

    def delete_test_server(self, server):
        self.os.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.os.servers_client, server['id'], True)

    def wait_for_notification(self, callback, *args):
        start_time = int(time.time())
        while True:
            if callback(*args):
                return

            if int(time.time()) - start_time >= self.notification_timeout:
                raise exceptions.TimeoutException

            time.sleep(self.notification_interval)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.api.auth import mixed_auth
from almanach.core import exception
from almanach.tests.unit import base


class TestMixedAuthentication(base.BaseTestCase):

    def setUp(self):
        super(TestMixedAuthentication, self).setUp()
        self.auth_one = mock.Mock()
        self.auth_two = mock.Mock()
        self.auth_backend = mixed_auth.MixedAuthentication([self.auth_one, self.auth_two])

    def tearDown(self):
        super(TestMixedAuthentication, self).tearDown()

    def test_with_token_valid_with_auth_one(self):
        token = "my token"
        self.auth_one.validate.return_value = True

        self.assertEqual(self.auth_backend.validate(token), True)
        self.auth_one.validate.assert_called_once()

    def test_with_token_valid_with_auth_two(self):
        token = "my token"
        self.auth_one.validate.side_effect = exception.AuthenticationFailureException
        self.auth_two.validate.return_value = True

        self.assertEqual(self.auth_backend.validate(token), True)
        self.auth_one.validate.assert_called_once()
        self.auth_two.validate.assert_called_once()

    def test_with_token_valid_with_auth_twos(self):
        token = "bad token"
        self.auth_one.validate.side_effect = exception.AuthenticationFailureException
        self.auth_two.validate.side_effect = exception.AuthenticationFailureException

        self.assertRaises(exception.AuthenticationFailureException, self.auth_backend.validate, token)
        self.auth_one.validate.assert_called_once()
        self.auth_two.validate.assert_called_once()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_serialization import jsonutils as json
from oslo_utils import timeutils

from almanach.tests.tempest.tests.api import base


class TestVolumeDeletion(base.BaseAlmanachTest):

    @classmethod
    def resource_setup(cls):
        super(TestVolumeDeletion, cls).resource_setup()

    def test_delete_volume(self):
        resp, tenant_id, volume = self.create_volume()
        self.assertEqual(resp.status, 201)

        delete_body = {'date': timeutils.isotime()}
        resp, _ = self.almanach_client.delete_volume(volume['volume_id'], json.dumps(delete_body))
        self.assertEqual(resp.status, 202)

        resp, response_body = self.almanach_client.get_tenant_entities(tenant_id)
        response_body = json.loads(response_body)

        self.assertEqual(resp.status, 200)
        self.assertIsInstance(response_body, list)
        self.assertEqual(1, len(response_body))
        self.assertEqual(volume['volume_id'], response_body[0]['entity_id'])
        self.assertEqual('volume', response_body[0]['entity_type'])
        self.assertIsNotNone(response_body[0]['end'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log as logging
from oslo_service import service

from almanach.collector.filters import delete_instance_before_completion_filter
from almanach.collector.filters import errored_instance_filter
from almanach.collector.handlers import instance_handler
from almanach.collector.handlers import volume_handler
from almanach.collector.handlers import volume_type_handler
from almanach.collector import messaging
from almanach.collector import notification

LOG = logging.getLogger(__name__)


class CollectorService(service.ServiceBase):

    def __init__(self, listeners, thread_pool_size):
        super(CollectorService, self).__init__()
        self.listeners = listeners
        self.thread_pool_size = thread_pool_size

    def start(self):
        LOG.info('Starting collector listeners...')
        for listener in self.listeners:
            listener.start(override_pool_size=self.thread_pool_size)

    def wait(self):
        LOG.info('Waiting...')

    def stop(self):
        LOG.info('Graceful shutdown of the collector service...')
        for listener in self.listeners:
            listener.stop()

    def reset(self):
        pass


class ServiceFactory(object):

    def __init__(self, config, core_factory):
        self.config = config
        self.core_factory = core_factory

    def get_service(self):
        messaging_factory = messaging.MessagingFactory(self.config)

        notification_handler = notification.NotificationHandler(self.config, messaging_factory)
        notification_handler.add_event_handler(self.get_instance_handler())
        notification_handler.add_event_handler(self.get_volume_handler())
        notification_handler.add_event_handler(self.get_volume_type_handler())

        listeners = messaging_factory.get_listeners(notification_handler)
        return CollectorService(listeners, self.config.collector.thread_pool_size)

    def get_instance_handler(self):
        return instance_handler.InstanceHandler(
                self.core_factory.get_instance_controller(),
                self.get_on_delete_filters())

    def get_volume_handler(self):
        return volume_handler.VolumeHandler(self.core_factory.get_volume_controller())

    def get_volume_type_handler(self):
        return volume_type_handler.VolumeTypeHandler(self.core_factory.get_volume_type_controller())

    def get_on_delete_filters(self):
        filters = notification.NotificationFilter()
        filters.add(errored_instance_filter.ErroredInstanceFilter())
        filters.add(delete_instance_before_completion_filter.DeleteInstanceBeforeCompletionFilter(self.config))
        return filters



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils

from almanach.tests.tempest.tests.api import base


class TestServerDeletion(base.BaseAlmanachTest):

    def setUp(self):
        super(base.BaseAlmanachTest, self).setUp()

    @classmethod
    def resource_setup(cls):
        super(TestServerDeletion, cls).resource_setup()

    def test_instance_delete(self):
        tenant_id = uuidutils.generate_uuid()
        server = self.get_server_creation_payload()
        self.create_server_through_api(tenant_id, server)

        self.almanach_client.delete_server(server['id'],
                                           json.dumps({'date': '2016-01-01T19:50:00Z'}))
        _, response_body = self.almanach_client.get_tenant_entities(tenant_id)

        response_body = json.loads(response_body)
        self.assertIsInstance(response_body, list)
        self.assertEqual(1, len(response_body))

        self.assertEqual(server['id'], response_body[0]['entity_id'])
        self.assertIsNotNone(response_body[0]['end'])



# Copyright 2017 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime
import pytz

from almanach.core.helpers import date_helper

from almanach.tests.unit import base


class TestDateHelper(base.BaseTestCase):

    def setUp(self):
        super(TestDateHelper, self).setUp()
        self.helper = date_helper.DateHelper()

    def test_parser_with_datetime_object(self):
        original_datetime = datetime.now(tz=pytz.utc)
        d = self.helper.parse(original_datetime)
        self.assertEqual(original_datetime, d)

    def test_parser_created_at_string(self):
        d = self.helper.parse('2017-02-15 05:22:57+00:00')
        self.assertIsInstance(d, datetime)
        self.assertEqual(d.day, 15)
        self.assertEqual(d.month, 2)
        self.assertEqual(d.year, 2017)
        self.assertEqual(d.hour, 5)
        self.assertEqual(d.minute, 22)
        self.assertEqual(d.second, 57)
        self.assertEqual(d.tzinfo, pytz.utc)

    def test_parser_deleted_at_string(self):
        d = self.helper.parse('2017-02-05T11:34:36.000000')
        self.assertIsInstance(d, datetime)
        self.assertEqual(d.day, 5)
        self.assertEqual(d.month, 2)
        self.assertEqual(d.year, 2017)
        self.assertEqual(d.hour, 11)
        self.assertEqual(d.minute, 34)
        self.assertEqual(d.second, 36)
        self.assertEqual(d.tzinfo, pytz.utc)

    def test_is_whithin_range(self):
        d1 = self.helper.parse('2017-02-15 05:22:57+00:00')
        d2 = self.helper.parse('2017-02-15T05:30:04.000000')
        self.assertFalse(self.helper.is_within_range(d1, d2, 30))
        self.assertTrue(self.helper.is_within_range(d1, d2, 600))

        d2 = self.helper.parse('2017-02-15 05:22:57+00:00')
        d1 = self.helper.parse('2017-02-15T05:30:04.000000')
        self.assertFalse(self.helper.is_within_range(d1, d2, 30))
        self.assertTrue(self.helper.is_within_range(d1, d2, 600))



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector import notification
from almanach.tests.unit import base


class TestNotificationHandler(base.BaseTestCase):

    def setUp(self):
        super(TestNotificationHandler, self).setUp()

        self.config_fixture.config(retry_delay=0, group='collector')
        self.config_fixture.config(max_retries=3, group='collector')

        self.messaging = mock.Mock()
        self.notifier = mock.Mock()
        self.messaging.get_notifier.return_value = self.notifier
        self.handler = notification.NotificationHandler(self.config, self.messaging)

    def test_retry_counter(self):
        message = notification.NotificationMessage('event_type', dict(), dict(), dict())

        self.assertEqual(0, message.get_retry_counter())

        message.increment_retry_count()
        self.assertEqual(1, message.get_retry_counter())

        message.increment_retry_count()
        self.assertEqual(2, message.get_retry_counter())

    def test_incoming_notifications_are_processed(self):
        event_handler1 = mock.Mock()
        event_handler2 = mock.Mock()

        self.handler.add_event_handler(event_handler1)
        self.handler.add_event_handler(event_handler2)

        self.handler.info(dict(), 'compute.nova01', 'some_event', dict(), dict())

        event_handler1.handle_events.assert_called_once()
        event_handler2.handle_events.assert_called_once()

    def test_failed_notification_are_sent_to_error_queue(self):
        event_handler = mock.Mock()
        event_handler.handle_events.side_effect = Exception()

        self.handler.add_event_handler(event_handler)
        self.handler.info(dict(), 'compute.nova01', 'some_event', dict(), dict())

        self.notifier.error.assert_called_once()

        self.notifier.info.assert_not_called()
        self.notifier.critical.assert_not_called()

    def test_notifications_are_sent_again_to_info_queue_if_under_threshold(self):
        context = {
            notification.NotificationMessage.RETRY_COUNTER: 2
        }

        self.handler.error(context, 'compute.nova01', 'some_event', dict(), dict())
        self.notifier.info.assert_called_once()

        self.notifier.error.assert_not_called()
        self.notifier.critical.assert_not_called()

    def test_notifications_are_sent_to_critical_queue_if_above_threshold(self):
        context = {
            notification.NotificationMessage.RETRY_COUNTER: 3
        }

        event_handler = mock.Mock()
        event_handler.handle_events.side_effect = Exception()

        self.handler.add_event_handler(event_handler)

        self.handler.info(context, 'compute.nova01', 'some_event', dict(), dict())
        self.notifier.critical.assert_called_once()

        self.notifier.info.assert_not_called()
        self.notifier.error.assert_not_called()

    def test_unrelated_notifications_are_not_handled_in_error_queue(self):
        self.handler.error(dict(), 'compute.nova01', 'some_event', dict(), dict())
        self.notifier.info.assert_not_called()
        self.notifier.error.assert_not_called()
        self.notifier.critical.assert_not_called()



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from keystoneauth1 import exceptions as keystoneauth1_exceptions
from keystoneauth1.identity import v3
from keystoneauth1 import session
from keystoneclient.v3 import client as keystone_client

from almanach.api.auth import base_auth
from almanach.core import exception


class KeystoneAuthentication(base_auth.BaseAuth):

    def __init__(self, config):
        auth = v3.Password(username=config.keystone_authtoken.username,
                           password=config.keystone_authtoken.password,
                           user_domain_id=config.keystone_authtoken.user_domain_id,
                           user_domain_name=config.keystone_authtoken.user_domain_name,
                           project_domain_name=config.keystone_authtoken.project_domain_name,
                           project_name=config.keystone_authtoken.project_name,
                           auth_url=config.keystone_authtoken.auth_url)

        sess = session.Session(auth=auth)
        self._client = keystone_client.Client(session=sess)

    def validate(self, token):
        if token is None:
            raise exception.AuthenticationFailureException('No token provided')

        try:
            if not self._client.tokens.validate(token):
                raise exception.AuthenticationFailureException('Invalid token')
        except keystoneauth1_exceptions.HttpError as e:
            raise exception.AuthenticationFailureException(e.message)

        return True



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.collector.handlers import volume_type_handler
from almanach.tests.unit import base
from almanach.tests.unit.builders import notification as builder


class TestVolumeTypeHandler(base.BaseTestCase):

    def setUp(self):
        super(TestVolumeTypeHandler, self).setUp()
        self.controller = mock.Mock()
        self.volume_type_handler = volume_type_handler.VolumeTypeHandler(self.controller)

    def test_volume_type_created(self):
        notification = builder.VolumeTypeNotificationBuilder()\
            .with_event_type('volume_type.create')\
            .build()

        self.volume_type_handler.handle_events(notification)
        self.controller.create_volume_type.assert_called_once_with(
            notification.payload['volume_types']['id'],
            notification.payload['volume_types']['name']
        )



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_serialization import jsonutils as json
from oslo_utils import uuidutils
from tempest.lib import exceptions

from almanach.tests.tempest.tests.api import base


class TestServerCreation(base.BaseAlmanachTest):
    tenant_id = ''

    def setUp(self):
        super(base.BaseAlmanachTest, self).setUp()
        self.tenant_id = uuidutils.generate_uuid()

    @classmethod
    def resource_setup(cls):
        super(TestServerCreation, cls).resource_setup()

    def test_instance_create(self):
        server = self.get_server_creation_payload()

        resp = self.create_server_through_api(self.tenant_id, server)
        self.assertEqual(resp.status, 201)

        _, response_body = self.almanach_client.get_tenant_entities(self.tenant_id)

        response_body = json.loads(response_body)
        self.assertIsInstance(response_body, list)
        self.assertEqual(1, len(response_body))

        self.assertEqual(server['id'], response_body[0]['entity_id'])
        self.assertIsNone(response_body[0]['end'])

    def test_instance_create_bad_date_format(self):
        server = self.get_server_creation_payload()
        server['created_at'] = 'not_a_real_date'
        exception = None
        try:
            self.create_server_through_api(self.tenant_id, server)
        except exceptions.BadRequest as bad_request:
            exception = bad_request

        self.assertIsNotNone(exception)
        self.assertEqual('400', exception.resp['status'])
        self.assertIn('The provided date has an invalid format. Format should be of yyyy-mm-ddThh:mm:ss.msZ',
                      exception.resp_body['error'])



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from almanach.core import exception
from almanach.tests.unit.api.v1 import base_api
from oslo_utils import uuidutils


class TestApiVolume(base_api.BaseApi):

    def test_successful_volume_create(self):
        data = dict(volume_id="VOLUME_ID",
                    start="START_DATE",
                    volume_type="VOLUME_TYPE",
                    size="A_SIZE",
                    volume_name="VOLUME_NAME",
                    attached_to=["INSTANCE_ID"])

        code, result = self.api_post(
            '/project/PROJECT_ID/volume',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.volume_ctl.create_volume.assert_called_once_with(
            project_id="PROJECT_ID",
            **data
        )
        self.assertEqual(code, 201)

    def test_volume_create_missing_a_param_returns_bad_request_code(self):
        data = dict(volume_id="VOLUME_ID",
                    start="START_DATE",
                    size="A_SIZE",
                    volume_name="VOLUME_NAME",
                    attached_to=[])

        code, result = self.api_post(
            '/project/PROJECT_ID/volume',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.assertIn(
            "The 'volume_type' param is mandatory for the request you have made.",
            result["error"]
        )
        self.volume_ctl.create_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_create_bad_date_format_returns_bad_request_code(self):
        self.volume_ctl.create_volume.side_effect = exception.DateFormatException
        data = dict(volume_id="VOLUME_ID",
                    start="A_BAD_DATE",
                    volume_type="VOLUME_TYPE",
                    size="A_SIZE",
                    volume_name="VOLUME_NAME",
                    attached_to=["INSTANCE_ID"])

        code, result = self.api_post(
            '/project/PROJECT_ID/volume',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.volume_ctl.create_volume.assert_called_once_with(
            project_id="PROJECT_ID",
            **data
        )
        self.assertEqual(code, 400)

    def test_successful_volume_delete(self):
        data = dict(date="DELETE_DATE")

        code, result = self.api_delete('/volume/VOLUME_ID', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_ctl.delete_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            delete_date=data['date']
        )
        self.assertEqual(code, 202)

    def test_volume_delete_missing_a_param_returns_bad_request_code(self):
        code, result = self.api_delete('/volume/VOLUME_ID', data=dict(), headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The 'date' param is mandatory for the request you have made.",
            result["error"]
        )
        self.volume_ctl.delete_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_delete_no_data_bad_request_code(self):
        code, result = self.api_delete('/volume/VOLUME_ID', headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "Invalid parameter or payload",
            result["error"]
        )
        self.volume_ctl.delete_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_delete_bad_date_format_returns_bad_request_code(self):
        self.volume_ctl.delete_volume.side_effect = exception.DateFormatException
        data = dict(date="A_BAD_DATE")

        code, result = self.api_delete('/volume/VOLUME_ID', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.volume_ctl.delete_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            delete_date=data['date']
        )
        self.assertEqual(code, 400)

    def test_successful_volume_resize(self):
        data = dict(date="UPDATED_AT",
                    size="NEW_SIZE")

        code, result = self.api_put('/volume/VOLUME_ID/resize', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_ctl.resize_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            size=data['size'],
            update_date=data['date']
        )
        self.assertEqual(code, 200)

    def test_volume_resize_missing_a_param_returns_bad_request_code(self):
        data = dict(date="A_DATE")

        code, result = self.api_put('/volume/VOLUME_ID/resize', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The 'size' param is mandatory for the request you have made.",
            result["error"]
        )
        self.volume_ctl.resize_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_resize_bad_date_format_returns_bad_request_code(self):
        self.volume_ctl.resize_volume.side_effect = exception.DateFormatException
        data = dict(date="BAD_DATE",
                    size="NEW_SIZE")

        code, result = self.api_put('/volume/VOLUME_ID/resize', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.volume_ctl.resize_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            size=data['size'],
            update_date=data['date']
        )
        self.assertEqual(code, 400)

    def test_successful_volume_attach(self):
        data = dict(date="UPDATED_AT",
                    attachments=[uuidutils.generate_uuid()])

        code, result = self.api_put('/volume/VOLUME_ID/attach', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_ctl.attach_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            attachments=data['attachments'],
            date=data['date']
        )
        self.assertEqual(code, 200)

    def test_volume_attach_missing_a_param_returns_bad_request_code(self):
        data = dict(date="A_DATE")

        code, result = self.api_put(
            '/volume/VOLUME_ID/attach',
            data=data,
            headers={'X-Auth-Token': 'some token value'}
        )

        self.assertIn(
            "The 'attachments' param is mandatory for the request you have made.",
            result["error"]
        )
        self.volume_ctl.attach_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_attach_bad_date_format_returns_bad_request_code(self):
        self.volume_ctl.attach_volume.side_effect = exception.DateFormatException
        data = dict(date="A_BAD_DATE",
                    attachments=[uuidutils.generate_uuid()])

        code, result = self.api_put('/volume/VOLUME_ID/attach', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.volume_ctl.attach_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            attachments=data['attachments'],
            date=data['date']
        )
        self.assertEqual(code, 400)

    def test_successful_volume_detach(self):
        data = dict(date="UPDATED_AT",
                    attachments=[uuidutils.generate_uuid()])

        code, result = self.api_put('/volume/VOLUME_ID/detach', data=data, headers={'X-Auth-Token': 'some token value'})

        self.volume_ctl.detach_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            attachments=data['attachments'],
            date=data['date']
        )
        self.assertEqual(code, 200)

    def test_volume_detach_missing_a_param_returns_bad_request_code(self):
        data = dict(date="A_DATE")

        code, result = self.api_put('/volume/VOLUME_ID/detach', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The 'attachments' param is mandatory for the request you have made.",
            result["error"]
        )
        self.volume_ctl.detach_volume.assert_not_called()
        self.assertEqual(code, 400)

    def test_volume_detach_bad_date_format_returns_bad_request_code(self):
        self.volume_ctl.detach_volume.side_effect = exception.DateFormatException
        data = dict(date="A_BAD_DATE",
                    attachments=[uuidutils.generate_uuid()])

        code, result = self.api_put('/volume/VOLUME_ID/detach', data=data, headers={'X-Auth-Token': 'some token value'})

        self.assertIn(
            "The provided date has an invalid format. "
            "Format should be of yyyy-mm-ddThh:mm:ss.msZ, ex: 2015-01-31T18:24:34.1523Z",
            result["error"]
        )
        self.volume_ctl.detach_volume.assert_called_once_with(
            volume_id="VOLUME_ID",
            attachments=data['attachments'],
            date=data['date']
        )
        self.assertEqual(code, 400)



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from almanach.tests.unit.api.v1 import base_api


class TestApiInfo(base_api.BaseApi):

    def test_info(self):
        info = {'info': {'version': '1.0'}, 'database': {'all_entities': 10, 'active_entities': 2}}
        self.app_ctl.get_application_info.return_value = info

        code, result = self.api_get('/info')

        self.app_ctl.get_application_info.assert_called_once()
        self.assertEqual(code, 200)
        self.assertIn('info', result)
        self.assertEqual(result['info']['version'], '1.0')



# Copyright 2016 Internap.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from almanach.core.controllers import application_controller
from almanach.tests.unit import base


class TestApplicationController(base.BaseTestCase):

    def setUp(self):
        super(TestApplicationController, self).setUp()
        self.database_adapter = mock.Mock()
        self.controller = application_controller.ApplicationController(self.database_adapter)

    def test_get_application_info(self):
        self.database_adapter.count_entities.return_value = 20
        self.database_adapter.count_active_entities.return_value = 16

        info = self.controller.get_application_info()

        self.assertIsNotNone(info['info']['version'])
        self.assertEqual(20, info['database']['all_entities'])
        self.assertEqual(16, info['database']['active_entities'])

        self.database_adapter.count_entities.assert_called_once()
        self.database_adapter.count_active_entities.assert_called_once()
